#pragma once

#include "ui.h"

namespace kek {

class UIWindow: public UIElement {

	UIWindow(UIValue x, UIValue y);

	virtual ~UIWindow();

	virtual UIElementType getType();

	virtual void draw(UIPoint screenPos, glm::mat4 projection);

};

class TextElement: public UIElement {

protected:
	TextObject *text;

public:
	int sizePixels;
	Color color;
	TextMode textMode;
	TextBounds textBounds;

	TextElement(UIValue x, UIValue y, Font *font);

	TextElement(UIValue x, UIValue y);

	virtual ~TextElement();

	virtual UIElementType getType();

	virtual UIBounds getBounds();

	void setText(std::string text);

	virtual void draw(UIPoint screenPos, glm::mat4 projection);

};

class RectangleElement: public UIElement {

protected:
	UIValue w, h;
	Color color;

	unsigned int vao;
	unsigned int vbo;

public:
	RectangleElement(UIValue x, UIValue y, UIValue w, UIValue h);

	virtual ~RectangleElement();

	virtual UIElementType getType();

	virtual UIBounds getBounds();

	virtual void setColor(Color color);

	virtual void draw(UIPoint screenPos, glm::mat4 projection);

};

class ButtonElement: public RectangleElement {

protected:
	TextElement *text;
	Color color;
	Callable onClick;

public:
	ButtonElement(UIValue x, UIValue y, UIValue w, UIValue h);

	virtual ~ButtonElement();

	virtual UIElementType getType();

	TextElement *getText();

	virtual void setColor(Color color);

	void setOnClickCallback(Callable onClick);

	virtual void hover(UIPoint pos, UIPoint screenPos);

	virtual void hoverEnter();

	virtual void hoverExit();

	virtual void click(UIPoint pos, UIPoint screenPos, GLFWMouseButton button);

};

}
