#include <iostream>

#include "kekengine.h"
#include <GL/glew.h>
#include <GLFW/glfw3.h>

using namespace kek;

static KeyBinding keyWow;

void periodicCallback(GLFWwindow *window, void *data){
	if(Input::getKeyState(keyWow) == GLFW_PRESS) {
		//std::cout << "WOW" << std::endl;
	}
}

void keyCallback(GLFWwindow *window, int key, int scancode, int action, int mods, void *data){
	if(key == Input::getKeyBinding(keyWow).key && action == GLFW_PRESS) {
		std::cout << "WOW" << std::endl;
		if(glfwGetInputMode(window, GLFW_CURSOR) == GLFW_CURSOR_DISABLED) {
			glfwSetInputMode(window, GLFW_CURSOR, GLFW_CURSOR_NORMAL);
		}else {
			glfwSetInputMode(window, GLFW_CURSOR, GLFW_CURSOR_DISABLED);
		}
	}
}

void mouseButtonCallback(GLFWwindow *window, int button, int action, int mods, void *data){
	std::cout << "HELLO" << std::endl;
}

int main(int argc, char **argv) {
	Engine::init();

	MemoryBuffer *buf = Resource::loadResource("object/sphere/Sphere.obj");
	Mesh *mesh = ObjParser::parseMesh(buf, "object/sphere/");
	delete buf;

	GameObject *test = new GameObject();
	test->addMesh(mesh);

	Scene *scene = new Scene();
	scene->addObject(test);

	for(int i = 0; i < 10; i++) {
		GameObject *test2 = new GameObject();
		test2->addMesh(ObjParser::loadMesh("object/cube_colored/Cube.obj"));
		test2->moveTo(glm::vec3(0.0f, 5.0f, 3 * i));
		scene->addObject(test2);
	}

	PointLight *light = new PointLight(glm::vec3(1), 1, 0, 1);
	scene->lights->add(light);

	//DirectionalLight *l = new DirectionalLight(glm::vec3(1), glm::vec3(1, -1, 1));
	//scene->lights->add(l);

	//SpotLight *spot = new SpotLight(glm::vec3(1), glm::vec3(0, 0, -1), 1, 0, 1, glm::radians(8.0f), glm::radians(15.0f));
	//spot->moveTo(glm::vec3(0, 0, 5));
	//scene->lights->add(spot);

	Engine::setActiveScene(scene);

	keyWow = Input::createKeyBinding("wow", GLFW_KEY_O);
	Input::addPeriodicCallback(PeriodicCallback(periodicCallback, nullptr));
	Input::addKeyListener(KeyCallback(keyCallback, nullptr));
	Input::addMouseButtonListener(MouseButtonCallback(mouseButtonCallback, nullptr));

	ButtonElement *btn = new ButtonElement(px(0), px(100), px(200), px(50));
	btn->getText()->setText("Hello There!");
	UI::addElement(btn);

	Engine::start();
}
